package joinexistentcontingencygroup;

import java.util.concurrent.ConcurrentLinkedQueue;
import com.fxcore2.*;
import java.util.List;

public class EventListener {
    private SimpleLog mSimpleLog;
    private ConcurrentLinkedQueue<O2GResponse> mResponseQueue;
    private O2GSession mSession;
    private EventPipe mEventPipe;

    class ParamsFromLoginRules {
        String sAccountID;
        String sOfferID;
        int iBaseAmount;
        double dPointSize;
        double dAskRate;
        double dBidRate;
    }
    public EventListener(SimpleLog simpleLog, ConcurrentLinkedQueue<O2GResponse> responseQueue, O2GSession session, EventPipe eventPipe) {
        mSimpleLog = simpleLog;
        mResponseQueue = responseQueue;
        mSession = session;
        mEventPipe = eventPipe;
    }

    public void waitNextEvent() {
        boolean bContinue = true;
        while (bContinue) {
            EventType eEventType = mEventPipe.waitEvent();
            if (eEventType == EventType.STATUS) {
                this.onStatusChanged();
            } else if (eEventType == EventType.CONNECTED) {
                this.onConnected();
            } else if (eEventType == EventType.RESPONSE) {
                this.onResponse();
            } else {
                // "How to login" : Logout when you are done.
                mSession.logout();
                bContinue = false;
            }
        }
    }

    private void onResponse() {
        while (true) {
            if (mResponseQueue.size() == 0)
                break;

            O2GResponse response = mResponseQueue.poll();

        }
    }

    private void onConnected() {
        ParamsFromLoginRules params = new ParamsFromLoginRules();
        boolean bParamsOK = prepareParamsFromLoginRules(params);
        if (!bParamsOK)
            return;

        joinExistingContingencyGroup(params.sAccountID, Main.sPrimaryID, Main.listSecondary, Main.iContingencyType); 
    }

    private void onStatusChanged() {
    }

    private void joinExistingContingencyGroup(String sAccountID, String sPrimaryID, List<String> listSecondary, int iContingencyType)
    {
        O2GRequestFactory factory = mSession.getRequestFactory();
        O2GValueMap valuemap = factory.createValueMap();
        valuemap.setString(O2GRequestParamsEnum.COMMAND, Constants.Commands.JoinToExistingContingencyGroup);
        valuemap.setInt(O2GRequestParamsEnum.CONTINGENCY_GROUP_TYPE, iContingencyType);
        valuemap.setString(O2GRequestParamsEnum.OCO_BULK_ID, sPrimaryID);
        for (int k = 0; k < listSecondary.size(); k++)
        {
            valuemap.appendChild(createValueMapForExistingOrder(listSecondary.get(k), sAccountID)); // secondary order
        }
        
        // create request from valueMap
        O2GRequest request = factory.createOrderRequest(valuemap);
        mSession.sendRequest(request);
    }

    public O2GValueMap createValueMapForExistingOrder(String sOrderID, String sAccountID)
    {
      O2GRequestFactory factory = mSession.getRequestFactory();
      O2GValueMap valuemap = factory.createValueMap();
      valuemap.setString(O2GRequestParamsEnum.ORDER_ID, sOrderID);
      valuemap.setString(O2GRequestParamsEnum.ACCOUNT_ID, sAccountID);
      return valuemap;
    }
    private boolean prepareParamsFromLoginRules(ParamsFromLoginRules params)
    {
        boolean bOK = false;
        O2GResponseReaderFactory factory = mSession.getResponseReaderFactory();
        O2GLoginRules loginRules = mSession.getLoginRules();
        // get first account from login
        O2GResponse accountResponse = loginRules.getTableRefeshResponse(O2GTable.ACCOUNTS);
        O2GAccountsTableResponseReader accountsReader = factory.createAccountsTableReader(accountResponse);
        O2GAccountRow account = accountsReader.getRow(0);
        // get account id
        params.sAccountID = account.getAccountID();
        // get base amount
        params.iBaseAmount = account.getBaseUnitSize();
        params.sOfferID = "";
        params.dAskRate = 0;
        params.dBidRate = 0;
        params.dPointSize = 0;
        // Get offers for EUR/USD
        O2GResponse offersResponse = loginRules.getTableRefeshResponse(O2GTable.OFFERS);
        O2GOffersTableResponseReader offersReader = factory.createOffersTableReader(offersResponse);
        for (int i = 0; i < offersReader.size(); i++)
        {
            O2GOfferRow offer = offersReader.getRow(i);
            if (offer.getInstrument().equals("EUR/USD"))
            {
                params.sOfferID = offer.getOfferID();
                params.dAskRate = offer.getAsk();
                params.dBidRate = offer.getBid();
                params.dPointSize = offer.getPointSize();
                bOK = true;
                break;
            }
        }
        return bOK;
    }


}